#include "Graphics/Renderer.h"

BarinkEngine::Renderer::Renderer()
{
	models = std::vector<GPU_Bucket*>();
}

BarinkEngine::Renderer::~Renderer()
{
	// CleanUp!

	// For each model submitted 
	for ( auto packet : models )
	{
		delete packet;
	}

	// glDeleteBuffers(1, &UV_id);
}

void BarinkEngine::Renderer::Render()
{
	// This creation of the projection and camera is somewhat wastefull
	Camera cam =  Camera(glm::vec3(8.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.0f), 90.0f);
	glm::mat4 projection = glm::perspective(glm::radians(cam.Zoom), (800.0f / 600.0f), 0.001f, 100.0f);

	
	for (auto model : models) {
		// Push matrices etc ....
		model->vertexarray.Bind();

		model->elementBuffer.Bind(true);
		

		if (model->material == nullptr) {
			std::cout << "No material attached!" << std::endl;
		}
		else {
			model->material->shader.Use();
			model->material->Apply();

			model->material->shader.setUniformVec3("Color", model->material->Color);
			model->material->shader.setUniformMat4("M", glm::mat4());
			model->material->shader.setUniformMat4("V", cam.GetViewMatrix());
			model->material->shader.setUniformMat4("P", projection);
		
		}
		
		// Update perf counters 
		ES->verts = model->mesh->vertices.size();
		ES->DC++;

		glDrawElements(GL_TRIANGLES,
			static_cast<unsigned int>(model->mesh->elements.size()),
			GL_UNSIGNED_INT,
			NULL
		);
		model->vertexarray.Unbind();

	
		
	}


}


// Upload data to cpu and add object to render list
void BarinkEngine::Renderer::Submit(Renderable* model)
{
	// Upload mesh data to gpu for render  
	
	GPU_Bucket* bucket = new GPU_Bucket();

	bucket->material = model->material;
	
	bucket->Upload(*model->mesh);
	
	models.push_back(bucket);
	
}
